@@ -27,9 +27,6 @@ char *alloca ();
 #endif
 #endif /* TRE_USE_ALLOCA */
 
-#define __USE_STRING_INLINES
-#undef __NO_INLINE__
-
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,7 +46,6 @@ char *alloca ();
 
 #include "tre-internal.h"
 #include "tre-match-utils.h"
-#include "tre.h"
 #include "xmalloc.h"
 
 #define TRE_M_COST	0
@@ -323,7 +319,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
   GET_NEXT_WCHAR();
   pos = 0;
 
-  while (/*CONSTCOND*/1)
+  while (/*CONSTCOND*/(void)1,1)
     {
       DPRINT(("%03d:%2lc/%05d\n", pos, (tre_cint_t)next_c, (int)next_c));
 
@@ -762,10 +758,10 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 			       default_params);
 
 	      /* Set the costs after this transition. */
-		memcpy(&reach_next[dest_id].costs,
-		       reach[id].costs,
-		       sizeof(reach[id].costs[0][0])
-		       * TRE_M_LAST * (depth + 1));
+	      memcpy(&reach_next[dest_id].costs,
+		     reach[id].costs,
+		     sizeof(reach[id].costs[0][0])
+		     * TRE_M_LAST * (depth + 1));
 	      reach_next[dest_id].costs[depth][TRE_M_COST] = cost;
 	      reach_next[dest_id].costs[depth][TRE_M_NUM_SUBST] += err;
 	      reach_next[dest_id].costs[depth][TRE_M_NUM_ERR] += err;
