@@ -1,21 +1,8 @@
 /*
   tre-stack.c - Simple stack implementation
 
-  Copyright (c) 2001-2006 Ville Laurikari <vl@iki.fi>
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -25,16 +12,21 @@
 #include <stdlib.h>
 #include <assert.h>
 
-#include "tre-stack.h"
 #include "tre-internal.h"
+#include "tre-stack.h"
 #include "xmalloc.h"
 
+union tre_stack_item {
+  void *voidptr_value;
+  int int_value;
+};
+
 struct tre_stack_rec {
   int size;
   int max_size;
   int increment;
   int ptr;
-  void **stack;
+  union tre_stack_item *stack;
 };
 
 
@@ -73,8 +65,8 @@ tre_stack_num_objects(tre_stack_t *s)
   return s->ptr;
 }
 
-reg_errcode_t
-tre_stack_push(tre_stack_t *s, void *value)
+static reg_errcode_t
+tre_stack_push(tre_stack_t *s, union tre_stack_item value)
 {
   if (s->ptr < s->size)
     {
@@ -90,7 +82,7 @@ tre_stack_push(tre_stack_t *s, void *value)
 	}
       else
 	{
-	  void **new_buffer;
+	  union tre_stack_item *new_buffer;
 	  int new_size;
 	  DPRINT(("tre_stack_push: trying to realloc more space\n"));
 	  new_size = s->size + s->increment;
@@ -112,10 +104,22 @@ tre_stack_push(tre_stack_t *s, void *value)
   return REG_OK;
 }
 
-void *
-tre_stack_pop(tre_stack_t *s)
-{
-  return s->stack[--s->ptr];
+#define define_pushf(typetag, type)  \
+  declare_pushf(typetag, type) {     \
+    union tre_stack_item item;	     \
+    item.typetag ## _value = value;  \
+    return tre_stack_push(s, item);  \
 }
 
+define_pushf(int, int)
+define_pushf(voidptr, void *)
+
+#define define_popf(typetag, type)		    \
+  declare_popf(typetag, type) {			    \
+    return s->stack[--s->ptr].typetag ## _value;    \
+  }
+
+define_popf(int, int)
+define_popf(voidptr, void *)
+
 /* EOF */
