@@ -13,7 +13,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <regex.h>
 #include <time.h>
 
 #undef MALLOC_DEBUGGING
@@ -21,10 +20,12 @@
 #include "xmalloc.h"
 #endif /* MALLOC_DEBUGGING */
 
+#include "tre-internal.h"
+
 #define REGEXP_MAX_LEN 16
 
 int
-main(int argc, char **argv)
+main(void)
 {
   int len, i, flags, n;
   char regex[50];
