@@ -13,6 +13,17 @@
    regexp implementation.
 */
 
+/*
+   2023/06 - Compilers now sometimes require the input string constants to be
+             properly encoded, but how they decide on which encoding (if any)
+             is poorly documented and different for different platforms.
+             The non-ASCII encoded strings are now guarded by #ifdefs with one
+             of the following values.  Define/undef whichever one(s) you need.
+   #define SRC_IN_ISO_8859_1
+   #define SRC_IN_UTF_8
+   #define SRC_IN_EUC_JP
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
@@ -23,10 +34,15 @@
 #include <string.h>
 #include <sys/types.h>
 #include <locale.h>
+/* look for getopt in order to use a -o option for output. */
+#if defined(HAVE_UNISTD_H)
+#include <unistd.h>
+#elif defined(HAVE_GETOPT_H)
+#include <getopt.h>
+#endif
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif /* HAVE_MALLOC_H */
-#include <regex.h>
 
 #ifdef TRE_VERSION
 #define HAVE_REGNEXEC 1
@@ -36,6 +52,7 @@
 #define xmalloc malloc
 #define xfree free
 #endif /* !TRE_VERSION */
+
 #include "tre-internal.h"
 
 #ifdef WRETEST
@@ -43,13 +60,27 @@
 #define CHAR_T wchar_t
 #define L(x) (L ## x)
 
-#define MAXSTRSIZE 1024
+#define MAXSTRSIZE 8192
 static wchar_t wstr[MAXSTRSIZE];
 static wchar_t wregex[MAXSTRSIZE];
 static int woffs[MAXSTRSIZE];
 
+#ifdef TRE_USE_SYSTEM_REGEX_H
+/* Avoid some redefinition warnings from including tre.h. */
+#ifdef tre_regexec
+#undef tre_regexec
+/* No need for the *n* fn, it isn't in the system abi. */
+#endif
+#endif
 #define tre_regexec tre_regwexec
 #define tre_regnexec tre_regwnexec
+#ifdef TRE_USE_SYSTEM_REGEX_H
+/* Avoid some redefinition warnings from including tre.h. */
+#ifdef tre_regcomp 
+#undef tre_regcomp
+#endif
+/* No need for the *n* fn, it isn't in the system abi. */
+#endif
 #define tre_regcomp tre_regwcomp
 #define tre_regncomp tre_regwncomp
 
@@ -102,6 +133,8 @@ mbntowc (wchar_t *buf, const char *str, size_t len, int *off)
 #define L(x) (x)
 #endif /* !WRETEST */
 
+static FILE *outf = NULL;
+
 static int valid_reobj = 0;
 static regex_t reobj;
 static regmatch_t pmatch_global[32];
@@ -126,10 +159,10 @@ static void
 test_status(char c)
 {
   static int k = 0;
-  printf("%c", c);
+  fprintf(outf, "%c", c);
   if (++k % 79 == 0)
-    printf("\n");
-  fflush(stdout);
+    fprintf(outf, "\n");
+  fflush(outf);
 }
 
 
@@ -227,15 +260,15 @@ check(va_list ap, int ret, const CHAR_T *str,
   if (ret != va_arg(ap, int))
     {
 #ifndef WRETEST
-      printf("Exec error, regex: \"%s\", cflags %d, "
+      fprintf(outf, "Exec error, regex: \"%s\", cflags %d, "
 	     "string: \"%s\", eflags %d\n", regex_pattern, cflags_global,
 	     str, eflags);
 #else /* WRETEST */
-      printf("Exec error, regex: \"%ls\", cflags %d, "
+      fprintf(outf, "Exec error, regex: \"%ls\", cflags %d, "
 	     "string: \"%ls\", eflags %d\n", regex_pattern, cflags_global,
 	     str, eflags);
 #endif /* WRETEST */
-      printf("	got %smatch (tre_regexec returned %d)\n", ret ? "no " : "", ret);
+      fprintf(outf, "	got %smatch (tre_regexec returned %d)\n", ret ? "no " : "", ret);
       return 1;
     }
 
@@ -258,7 +291,7 @@ check(va_list ap, int ret, const CHAR_T *str,
 	      if ((rm_so = woffs[rm_so]) < 0 ||
 		  (n = rm_eo, rm_eo = woffs[rm_eo]) < 0)
 		{
-		  printf("Invalid or incomplete multi-byte sequence "
+		  fprintf(outf, "Invalid or incomplete multi-byte sequence "
 			 "in string %ls before byte offset %d\n", str, n);
 		  return 1;
 		}
@@ -268,14 +301,14 @@ check(va_list ap, int ret, const CHAR_T *str,
 	      || pmatch[i].rm_eo != rm_eo)
 	    {
 #ifndef WRETEST
-	      printf("Exec error, regex: \"%s\", string: \"%s\"\n",
+	      fprintf(outf, "Exec error, regex: \"%s\", string: \"%s\"\n",
 		     regex_pattern, str);
-	      printf("	group %d: expected (%d, %d) \"%.*s\", "
+	      fprintf(outf, "	group %d: expected (%d, %d) \"%.*s\", "
 		     "got (%d, %d) \"%.*s\"\n",
 #else /* WRETEST */
-	      printf("Exec error, regex: \"%ls\", string: \"%ls\"\n",
+	      fprintf(outf, "Exec error, regex: \"%ls\", string: \"%ls\"\n",
 		     regex_pattern, str);
-	      printf("	group %d: expected (%d, %d) \"%.*ls\", "
+	      fprintf(outf, "	group %d: expected (%d, %d) \"%.*ls\", "
 		     "got (%d, %d) \"%.*ls\"\n",
 #endif /* WRETEST */
 		     i, rm_so, rm_eo, rm_eo - rm_so, str + rm_so,
@@ -290,11 +323,11 @@ check(va_list ap, int ret, const CHAR_T *str,
 	  && reobj.re_nsub <= pmatch_len && pmatch)
 	{
 #ifndef WRETEST
-	  printf("Comp error, regex: \"%s\"\n", regex_pattern);
+	  fprintf(outf, "Comp error, regex: \"%s\"\n", regex_pattern);
 #else /* WRETEST */
-	  printf("Comp error, regex: \"%ls\"\n", regex_pattern);
+	  fprintf(outf, "Comp error, regex: \"%ls\"\n", regex_pattern);
 #endif /* WRETEST */
-	  printf("  re_nsub is %d, should be %d\n", (int)reobj.re_nsub, i - 1);
+	  fprintf(outf, "  re_nsub is %d, should be %d\n", (int)reobj.re_nsub, i - 1);
 	  fail = 1;
 	}
 
@@ -304,13 +337,13 @@ check(va_list ap, int ret, const CHAR_T *str,
 	  {
 	    if (!fail)
 #ifndef WRETEST
-	      printf("Exec error, regex: \"%s\", string: \"%s\"\n",
+	      fprintf(outf, "Exec error, regex: \"%s\", string: \"%s\"\n",
 		     regex_pattern, str);
 #else /* WRETEST */
-	      printf("Exec error, regex: \"%ls\", string: \"%ls\"\n",
+	      fprintf(outf, "Exec error, regex: \"%ls\", string: \"%ls\"\n",
 		     regex_pattern, str);
 #endif /* WRETEST */
-	    printf("  group %d: expected (-1, -1), got (%d, %d)\n",
+	    fprintf(outf, "  group %d: expected (-1, -1), got (%d, %d)\n",
 		   i, (int)pmatch[i].rm_so, (int)pmatch[i].rm_eo);
 	    fail = 1;
 	  }
@@ -341,7 +374,7 @@ test_nexec(const char *data, size_t len, int eflags, ...)
 	if (wlen < 0)
 	  {
 	    exec_errors++;
-	    printf("Invalid or incomplete multi-byte sequence in %s\n", data);
+	    fprintf(outf, "Invalid or incomplete multi-byte sequence in %s\n", data);
 	    return;
 	  }
 	wstr[wlen] = L'\0';
@@ -415,7 +448,7 @@ test_exec(const char *str, int eflags, ...)
 	if (wlen < 0)
 	  {
 	    exec_errors++;
-	    printf("Invalid or incomplete multi-byte sequence in %s\n", str);
+	    fprintf(outf, "Invalid or incomplete multi-byte sequence in %s\n", str);
 	    return;
 	  }
 	wstr[wlen] = L'\0';
@@ -485,7 +518,7 @@ test_comp(const char *re, int flags, int ret)
     if (wlen < 0)
       {
 	comp_errors++;
-	printf("Invalid or incomplete multi-byte sequence in %s\n", re);
+	fprintf(outf, "Invalid or incomplete multi-byte sequence in %s\n", re);
 	return;
       }
     wregex[wlen] = L'\0';
@@ -497,7 +530,8 @@ test_comp(const char *re, int flags, int ret)
   cflags_global = flags;
 
 #ifdef MALLOC_DEBUGGING
-  {
+  xmalloc_configure(-1);
+  if (ret != REG_ESPACE) {
     static int j = 0;
     int i = 0;
     while (1)
@@ -517,10 +551,9 @@ test_comp(const char *re, int flags, int ret)
 #endif /* REGEX_DEBUG */
 	i++;
       }
-  }
-#else /* !MALLOC_DEBUGGING */
-  errcode = wrap_regcomp(&reobj, re, len, flags);
+  } else
 #endif /* !MALLOC_DEBUGGING */
+  errcode = wrap_regcomp(&reobj, re, len, flags);
 
 #ifdef WRETEST
 #undef re
@@ -529,11 +562,11 @@ test_comp(const char *re, int flags, int ret)
   if (errcode != ret)
     {
 #ifndef WRETEST
-      printf("Comp error, regex: \"%s\"\n", regex_pattern);
+      fprintf(outf, "Comp error, regex: \"%s\"\n", regex_pattern);
 #else /* WRETEST */
-      printf("Comp error, regex: \"%ls\"\n", regex_pattern);
+      fprintf(outf, "Comp error, regex: \"%ls\"\n", regex_pattern);
 #endif /* WRETEST */
-      printf("	expected return code %d, got %d.\n",
+      fprintf(outf, "	expected return code %d, got %d.\n",
 	     ret, errcode);
       comp_errors++;
     }
@@ -550,11 +583,32 @@ test_comp(const char *re, int flags, int ret)
 int
 main(int argc, char **argv)
 {
+  outf = stdout;
+#if defined(HAVE_UNISTD_H) || defined(HAVE_GETOPT_H)
+  int opt;
+  while ((opt = getopt(argc, argv, "o:")) != EOF)
+    {
+      switch (opt)
+	{
+	case 'o':
+	  if ((outf = fopen(optarg, "w")) == NULL)
+	    {
+	      perror(optarg);
+	      exit(1);
+	    }
+	  break;
+	default:
+	  /* getopt() will have printed an error message already */
+	  exit(1);
+	}
+    }
+#endif /* HAVE_UNISTD_H */
 
 #ifdef WRETEST
   /* Need an 8-bit locale.  Or move the two tests with non-ascii
      characters to the localized internationalization tests.  */
-  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") == NULL)
+  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") == NULL &&
+      setlocale(LC_CTYPE, "en_US.ISO8859-1") == NULL)
     fprintf(stderr, "Could not set locale en_US.ISO-8859-1.  Expect some\n"
 		    "`Invalid or incomplete multi-byte sequence' errors.\n");
 #endif /* WRETEST */
@@ -1032,7 +1086,7 @@ main(int argc, char **argv)
   test_comp("[^[:digit:]]+", REG_EXTENDED, 0);
   test_exec("%abC123890xyz=", 0, REG_OK, 0, 4, END);
   test_comp("[[:print:]]+", REG_EXTENDED, 0);
-  test_exec("\n %abC12\f", 0, REG_OK, 1, 8, END);
+  test_exec("\n\t %abC12\f", 0, REG_OK, 2, 9, END);
   test_comp("[[:upper:]]+", REG_EXTENDED, 0);
   test_exec("\n aBCDEFGHIJKLMNOPQRSTUVWXYz", 0, REG_OK, 3, 27, END);
   test_comp("[[:upper:]]+", REG_EXTENDED | REG_ICASE, 0);
@@ -1051,6 +1105,7 @@ main(int argc, char **argv)
   test_comp("[[:xdigit:]]+", REG_EXTENDED, 0);
   test_exec("-0123456789ABCDEFabcdef", 0, REG_OK, 1, 23, END);
   test_comp("[[:bogus-character-class-name:]", REG_EXTENDED, REG_ECTYPE);
+  test_comp("[[:\xff:", REG_EXTENDED, REG_ECTYPE);
 
 
   /* Range expressions (assuming that the C locale is being used). */
@@ -1290,7 +1345,19 @@ main(int argc, char **argv)
 
   /* Shorthands for character classes. */
   test_comp("\\w+", REG_EXTENDED, 0);
+#ifdef SRC_IN_ISO_8859_1
   test_exec(",.(a23_Nt-o)", 0, REG_OK, 3, 9, END);
+#else
+#ifdef SRC_IN_UTF_8
+  /* iconv -f ISO-8859-1 -t UTF-8 file_with_lines_above > www_utf_8 */
+  test_exec(",.(a23_Nt-öo)", 0, REG_OK, 3, 9, END);
+#else
+  unsigned char str_000[] = {
+    ',','.','(','a','2','3','_','N','t','-',0xF6,'o',0x00
+  };
+  test_exec((char const *)str_000, 0, REG_OK, 3, 9, END);
+#endif
+#endif
   test_comp("\\d+", REG_EXTENDED, 0);
   test_exec("uR120_4=v4", 0, REG_OK, 2, 5, END);
   test_comp("\\D+", REG_EXTENDED, 0);
@@ -1322,6 +1389,8 @@ main(int argc, char **argv)
   test_nexec("\000", 1, 0, REG_OK, 0, 1, END);
   test_comp("\\x{}r", REG_EXTENDED, 0);
   test_nexec("\000r", 2, 0, REG_OK, 0, 2, END);
+  test_comp("\\x{00000000}", REG_EXTENDED, 0);
+  test_comp("\\x{000000000}", REG_EXTENDED, REG_EBRACE);
 
   /* Tests for (?inrU-inrU) and (?inrU-inrU:) */
   test_comp("foo(?i)bar", REG_EXTENDED, 0);
@@ -1355,6 +1424,27 @@ main(int argc, char **argv)
   test_exec("parabc123wxyz", 0, REG_OK, 3, 13, END);
   test_exec("fooabc123wxyz", 0, REG_OK, 0, 3, END);
 
+  /*
+   * Test integer parser used for bounded repititions.
+   */
+
+  test_comp("a{9223372036854775808,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{9223372036854775808}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{9223372036854775807,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{9223372036854775807}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{2147483648,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{2147483648}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{2147483647,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{2147483647}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{32768,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{32768}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{32767,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{32767}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{256,}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{256}", REG_EXTENDED, REG_BADMAX);
+  test_comp("a{255,}", REG_EXTENDED, REG_OK);
+  test_comp("a{255}", REG_EXTENDED, REG_OK);
+
   /*
    * Test bounded repetitions.
    */
@@ -1393,6 +1483,21 @@ main(int argc, char **argv)
   test_exec("aaaaa", 0, REG_OK, 0, 5, END);
   test_exec("aaaaaa", 0, REG_OK, 0, 6, END);
   test_exec("aaaaaaa", 0, REG_OK, 0, 7, END);
+  test_comp("a{,}", REG_EXTENDED, REG_OK);
+  test_exec("", 0, REG_OK, 0, 0, END);
+  test_exec("aaa", 0, REG_OK, 0, 3, END);
+  test_comp("a{,0}", REG_EXTENDED, REG_OK);
+  test_exec("", 0, REG_OK, 0, 0, END);
+  test_exec("aaa", 0, REG_OK, 0, 0, END);
+  test_comp("a{,1}", REG_EXTENDED, REG_OK);
+  test_exec("", 0, REG_OK, 0, 0, END);
+  test_exec("a", 0, REG_OK, 0, 1, END);
+  test_exec("aa", 0, REG_OK, 0, 1, END);
+  test_comp("a{,2}", REG_EXTENDED, REG_OK);
+  test_exec("", 0, REG_OK, 0, 0, END);
+  test_exec("a", 0, REG_OK, 0, 1, END);
+  test_exec("aa", 0, REG_OK, 0, 2, END);
+  test_exec("aaa", 0, REG_OK, 0, 2, END);
 
   test_comp("a{5,10}", REG_EXTENDED, REG_OK);
   test_comp("a{6,6}", REG_EXTENDED, REG_OK);
@@ -1424,6 +1529,13 @@ main(int argc, char **argv)
   test_exec("abbabbbabaabbbbbbbbbbbba", 0, REG_OK,
 	    0, 24, 0, 10, 10, 22, END);
 
+  test_comp("^((a{1,2})?x)*y", REG_EXTENDED | REG_NOSUB, REG_OK);
+  test_exec("y", 0, REG_OK, END);
+  test_exec("xy", 0, REG_OK, END);
+  test_exec("axy", 0, REG_OK, END);
+  test_exec("aaxy", 0, REG_OK, END);
+  test_exec("aaaxy", 0, REG_NOMATCH, END);
+
   /* Test repeating something that has submatches inside. */
   test_comp("(a){0,5}", REG_EXTENDED, 0);
   test_exec("", 0, REG_OK, 0, 0, -1, -1, END);
@@ -1621,53 +1733,133 @@ main(int argc, char **argv)
   test_comp("a\\{1,0\\}", 0, REG_BADBR);
   test_comp("a\\{x\\}", 0, REG_BADBR);
   test_comp("a\\{\\}", 0, REG_BADBR);
+  test_comp("a\\{1,256\\}", 0, REG_BADMAX);
 
 
+#define TOOLONG 2048
+  static char toolong[TOOLONG + TOOLONG + 1];
+  memset(toolong, '(', TOOLONG);
+  memset(toolong + TOOLONG, ')', TOOLONG);
+  toolong[TOOLONG + TOOLONG] = '\0';
+  test_comp(toolong, REG_EXTENDED, REG_ESPACE);
 
 
   /*
    * Internationalization tests.
    */
 
-  /* This same test with the correct locale is below. */
+  /* This same test with the correct locale is below.
+     TBR: This is a guess for the source encoding, see comments below after the locale is set to a Japanese locale. */
+#ifdef SRC_IN_EUC_JP
   test_comp("+", REG_EXTENDED, 0);
-  test_exec("ξޤϡ", 0, REG_OK, 10, 13, END);
+  test_exec("ξޤϡ",
+            0, REG_OK, 10, 13, END);
+#else
+#ifdef SRC_IN_UTF_8
+  /* iconv -f EUC_JP -t UTF-8 file_with_lines_above > zzz_utf_8
+     This may be incorrect because the match results might be incorrect for UTF-8, I (TBR) just don't know enough to be certain.
+     It compiles and runs successfully on my desktop with the C.UTF-8 locale. */
+  test_comp("機+", REG_EXTENDED, 0);
+  test_exec("この賞は、機・利便性・セキ",
+            0, REG_OK, 15, 18, END);
+#else
+  /* Represent the test strings as a sequence of bytes so we don't run afoul of the compiler's expected source-charset. */
+  unsigned char str_001[] = {
+    0xB5,0xA1,'+',0x00
+  };
+  unsigned char str_002[] = {
+    0xA4,0xB3,0xA4,0xCE,0xBE,0xDE,0xA4,0xCF,0xA1,0xA2,0xB5,0xA1,0xA1,0xA6,0xCD,0xF8,0xCA,0xD8,0xC0,0xAD,0xA1,0xA6,0xA5,0xBB,0xA5,0xAD,0x00
+  };
+  test_comp((char const *)str_001, REG_EXTENDED, 0);
+  test_exec((char const *)str_002, 0, REG_OK, 10, 13, END);
+#endif
+#endif
 
 #if !defined(WIN32) && !defined(__OpenBSD__)
-  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") != NULL)
+  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") != NULL ||
+      setlocale(LC_CTYPE, "en_US.ISO8859-1") != NULL)
     {
-      printf("\nTesting LC_CTYPE en_US.ISO-8859-1\n");
+      fprintf(outf, "\nTesting LC_CTYPE en_US.ISO-8859-1\n");
+#ifdef SRC_IN_ISO_8859_1
       test_comp("aBCdeFghiJKlmnoPQRstuvWXyZ", REG_ICASE, 0);
       test_exec("abCDefGhiJKlmNoPqRStuVwXyz", 0, REG_OK, 0, 29, END);
+#else
+#ifdef SRC_IN_UTF_8
+      /* iconv -f ISO-8859-1 -t UTF-8 file_with_lines_above > yyy_utf_8 */
+      /* This fails with no match on freebsd, but succeeds in linux. */
+      test_comp("aBCdeFghiJKlmnoPQRstuvWXyZåäö", REG_ICASE, 0);
+      test_exec("abCDefGhiJKlmNoPqRStuVwXyzÅÄÖ", 0, REG_OK, 0, 29, END);
+#else
+      /* Represent the test strings as a sequence of bytes so we don't run afoul of the compiler's expected source-charset. */
+      unsigned char str_003[] = {
+        'a','B','C','d','e','F','g','h','i','J','K','l','m','n','o','P','Q','R','s','t','u','v','W','X','y','Z',0xE5,0xE4,0xF6,0x00
+      };
+      unsigned char str_004[] = {
+        'a','b','C','D','e','f','G','h','i','J','K','l','m','N','o','P','q','R','S','t','u','V','w','X','y','z',0xC5,0xC4,0xD6,0x00
+      };
+      test_comp((char const *)str_003, REG_ICASE, 0);
+      test_exec((char const *)str_004, 0, REG_OK, 0, 29, END);
+#endif
+#endif
     }
 
 #ifdef TRE_MULTIBYTE
-  if (setlocale(LC_CTYPE, "ja_JP.eucjp") != NULL)
+  if (setlocale(LC_CTYPE, "ja_JP.eucjp") != NULL ||
+      setlocale(LC_CTYPE, "ja_JP.eucJP") != NULL)
     {
-      printf("\nTesting LC_CTYPE ja_JP.eucjp\n");
+      fprintf(outf, "\nTesting LC_CTYPE ja_JP.eucjp\n");
       /* I tried to make a test where implementations not aware of multibyte
 	 character sets will fail.  I have no idea what the japanese text here
 	 means, I took it from http://www.ipsec.co.jp/. */
+      /* TBR 2023/03/22: iconv has (at least) the following encoding names for Japanese:
+           EUC-JIS-2004 EUC-JISX0213
+           EUC-JP-MS EUCJP-MS EUCJP-OPEN EUCJP-WIN EUCJPMS
+           EUC-JP CSEUCPKDFMTJAPANESE EUCJP IBM-EUCJP
+           ISO-2022-JP-1 ISO2022-JP1
+           ISO-2022-JP-2 CSISO2022JP2 ISO2022-JP2 ISO-2022-JP-2004 ISO-2022-JP-3 ISO2022-JP2004 ISO2022-JP3
+           ISO-2022-JP CSISO2022JP ISO2022-JP
+         Both iconv arguments of EUC-JP and EUC-JP-MS produced the converted strings below,
+         all the others I tried resulted in invalid characters.  So guess at EUC-JP.
+         If anyone knows what the encoding actually was, feel free to let me know at tbr at acm dot org :). */
+#ifdef SRC_IN_EUC_JP
       test_comp("+", REG_EXTENDED, 0);
       test_exec("ξޤϡ", 0, REG_OK, 10, 12, END);
-
+#else
+#ifdef SRC_IN_UTF_8
+      /* iconv -f EUC_JP -t UTF-8 file_with_lines_above > zzz_utf_8
+         This may fail because the match results might be incorrect for UTF-8, I (TBR) just don't know enough to be certain.
+         It compiles and runs successfully on my desktop with the C.UTF-8 locale. */
+      test_comp("機+", REG_EXTENDED, 0);
+      test_exec("この賞は、機・利便性・セキ", 0, REG_OK, 10, 12, END);
+#else
+      /* Represent the test strings as a sequence of bytes so we don't run afoul of the compiler's expected source-charset. */
+      /* This test uses the same strings (str_001 and str_002) as above, now with a Japanese locale.
+         NOTE THE DIFFERENCE IN MATCH RESULTS - (10,13) earlier with the default locale, and (10,12) here with the Japanese locale. */
+      test_comp((char const *)str_001, REG_EXTENDED, 0);
+      test_exec((char const *)str_002, 0, REG_OK, 10, 12, END);
+#endif
+#endif
       test_comp("a", REG_EXTENDED, 0);
       test_nexec("foo\000bar", 7, 0, REG_OK, 5, 6, END);
       test_comp("c$", REG_EXTENDED, 0);
       test_exec("abc", 0, REG_OK, 2, 3, END);
     }
+  else
+    {
+      fprintf(outf, "\nTRE_MULTIBYTE enabled, but skipping LC_CTYPE ja_JP.eucJP (locale unavailable)\n");
+    }
 #endif /* TRE_MULTIBYTE */
 #endif
 
   tre_regfree(&reobj);
 
-  printf("\n");
+  fprintf(outf, "\n");
   if (comp_errors || exec_errors)
-    printf("%d (%d + %d) out of %d tests FAILED!\n",
+    fprintf(outf, "%d (%d + %d) out of %d tests FAILED!\n",
 	   comp_errors + exec_errors, comp_errors, exec_errors,
 	   comp_tests + exec_tests);
   else
-    printf("All %d tests passed.\n", comp_tests + exec_tests);
+    fprintf(outf, "All %d tests passed.\n", comp_tests + exec_tests);
 
 
 #ifdef MALLOC_DEBUGGING
